@@ -3,7 +3,7 @@
 oauthlib.oauth2.rfc6749.errors
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-Error used both by OAuth 2 clients and provicers to represent the spec
+Error used both by OAuth 2 clients and providers to represent the spec
 defined error responses for all four core grant types.
 """
 from __future__ import unicode_literals
@@ -14,6 +14,7 @@ from oauthlib.common import urlencode, add_params_to_uri
 class OAuth2Error(Exception):
     error = None
     status_code = 400
+    description = ''
 
     def __init__(self, description=None, uri=None, state=None, status_code=None,
                  request=None):
@@ -36,7 +37,12 @@ class OAuth2Error(Exception):
 
         request:  Oauthlib Request object
         """
-        self.description = description
+        self.description = description or self.description
+        message = '(%s) %s' % (self.error, self.description)
+        if request:
+            message += ' ' + repr(request)
+        super(OAuth2Error, self).__init__(message)
+
         self.uri = uri
         self.state = state
 
@@ -49,6 +55,8 @@ class OAuth2Error(Exception):
             self.scopes = request.scopes
             self.response_type = request.response_type
             self.grant_type = request.grant_type
+            if not state:
+                self.state = request.state
 
     def in_uri(self, uri):
         return add_params_to_uri(uri, self.twotuples)
@@ -100,6 +108,7 @@ class MissingTokenTypeError(OAuth2Error):
 
 
 class FatalClientError(OAuth2Error):
+
     """Errors during authorization where user should not be redirected back.
 
     If the request fails due to a missing, invalid, or mismatching
@@ -113,27 +122,36 @@ class FatalClientError(OAuth2Error):
     pass
 
 
-class InvalidRedirectURIError(FatalClientError):
-    error = 'invalid_redirect_uri'
+class InvalidRequestFatalError(FatalClientError):
+    """For fatal errors, the request is missing a required parameter, includes
+    an invalid parameter value, includes a parameter more than once, or is
+    otherwise malformed.
+    """
+    error = 'invalid_request'
 
 
-class MissingRedirectURIError(FatalClientError):
-    error = 'missing_redirect_uri'
+class InvalidRedirectURIError(InvalidRequestFatalError):
+    description = 'Invalid redirect URI.'
 
 
-class MismatchingRedirectURIError(FatalClientError):
-    error = 'mismatching_redirect_uri'
+class MissingRedirectURIError(InvalidRequestFatalError):
+    description = 'Missing redirect URI.'
 
 
-class MissingClientIdError(FatalClientError):
-    error = 'invalid_client_id'
+class MismatchingRedirectURIError(InvalidRequestFatalError):
+    description = 'Mismatching redirect URI.'
 
 
-class InvalidClientIdError(FatalClientError):
-    error = 'invalid_client_id'
+class InvalidClientIdError(InvalidRequestFatalError):
+    description = 'Invalid client_id parameter value.'
+
+
+class MissingClientIdError(InvalidRequestFatalError):
+    description = 'Missing client_id parameter.'
 
 
 class InvalidRequestError(OAuth2Error):
+
     """The request is missing a required parameter, includes an invalid
     parameter value, includes a parameter more than once, or is
     otherwise malformed.
@@ -141,13 +159,19 @@ class InvalidRequestError(OAuth2Error):
     error = 'invalid_request'
 
 
+class MissingResponseTypeError(InvalidRequestError):
+    description = 'Missing response_type parameter.'
+
+
 class AccessDeniedError(OAuth2Error):
+
     """The resource owner or authorization server denied the request."""
     error = 'access_denied'
     status_code = 401
 
 
 class UnsupportedResponseTypeError(OAuth2Error):
+
     """The authorization server does not support obtaining an authorization
     code using this method.
     """
@@ -155,12 +179,14 @@ class UnsupportedResponseTypeError(OAuth2Error):
 
 
 class InvalidScopeError(OAuth2Error):
+
     """The requested scope is invalid, unknown, or malformed."""
     error = 'invalid_scope'
     status_code = 401
 
 
 class ServerError(OAuth2Error):
+
     """The authorization server encountered an unexpected condition that
     prevented it from fulfilling the request.  (This error code is needed
     because a 500 Internal Server Error HTTP status code cannot be returned
@@ -170,6 +196,7 @@ class ServerError(OAuth2Error):
 
 
 class TemporarilyUnavailableError(OAuth2Error):
+
     """The authorization server is currently unable to handle the request
     due to a temporary overloading or maintenance of the server.
     (This error code is needed because a 503 Service Unavailable HTTP
@@ -179,6 +206,7 @@ class TemporarilyUnavailableError(OAuth2Error):
 
 
 class InvalidClientError(OAuth2Error):
+
     """Client authentication failed (e.g. unknown client, no client
     authentication included, or unsupported authentication method).
     The authorization server MAY return an HTTP 401 (Unauthorized) status
@@ -194,6 +222,7 @@ class InvalidClientError(OAuth2Error):
 
 
 class InvalidGrantError(OAuth2Error):
+
     """The provided authorization grant (e.g. authorization code, resource
     owner credentials) or refresh token is invalid, expired, revoked, does
     not match the redirection URI used in the authorization request, or was
@@ -204,6 +233,7 @@ class InvalidGrantError(OAuth2Error):
 
 
 class UnauthorizedClientError(OAuth2Error):
+
     """The authenticated client is not authorized to use this authorization
     grant type.
     """
@@ -212,6 +242,7 @@ class UnauthorizedClientError(OAuth2Error):
 
 
 class UnsupportedGrantTypeError(OAuth2Error):
+
     """The authorization grant type is not supported by the authorization
     server.
     """
@@ -219,6 +250,7 @@ class UnsupportedGrantTypeError(OAuth2Error):
 
 
 class UnsupportedTokenTypeError(OAuth2Error):
+
     """The authorization server does not support the revocation of the
     presented token type.  I.e. the client tried to revoke an access token
     on a server not supporting this feature.
@@ -226,6 +258,60 @@ class UnsupportedTokenTypeError(OAuth2Error):
     error = 'unsupported_token_type'
 
 
+class FatalOpenIDClientError(FatalClientError):
+    pass
+
+class OpenIDClientError(OAuth2Error):
+    pass
+
+
+class InteractionRequired(OpenIDClientError):
+    """The Authorization Server requires End-User interaction to proceed.
+
+    This error MAY be returned when the prompt parameter value in the
+    Authentication Request is none, but the Authentication Request cannot be
+    completed without displaying a user interface for End-User interaction.
+    """
+    error = 'interaction_required'
+    status_code = 401
+
+
+class LoginRequired(OpenIDClientError):
+    """The Authorization Server requires End-User authentication.
+
+    This error MAY be returned when the prompt parameter value in the
+    Authentication Request is none, but the Authentication Request cannot be
+    completed without displaying a user interface for End-User authentication.
+    """
+    error = 'login_required'
+    status_code = 401
+
+
+class AccountSelectionRequried(OpenIDClientError):
+    """The End-User is REQUIRED to select a session at the Authorization Server.
+
+    The End-User MAY be authenticated at the Authorization Server with
+    different associated accounts, but the End-User did not select a session.
+    This error MAY be returned when the prompt parameter value in the
+    Authentication Request is none, but the Authentication Request cannot be
+    completed without displaying a user interface to prompt for a session to
+    use.
+    """
+    error = 'account_selection_required'
+
+
+class ConsentRequired(OpenIDClientError):
+
+    """The Authorization Server requires End-User consent.
+
+    This error MAY be returned when the prompt parameter value in the
+    Authentication Request is none, but the Authentication Request cannot be
+    completed without displaying a user interface for End-User consent.
+    """
+    error = 'consent_required'
+    status_code = 401
+
+
 def raise_from_error(error, params=None):
     import inspect
     import sys
@@ -234,6 +320,6 @@ def raise_from_error(error, params=None):
         'uri': params.get('error_uri'),
         'state': params.get('state')
     }
-    for _, cls in  inspect.getmembers(sys.modules[__name__], inspect.isclass):
+    for _, cls in inspect.getmembers(sys.modules[__name__], inspect.isclass):
         if cls.error == error:
             raise cls(**kwargs)
