@@ -6,7 +6,6 @@ This module contains methods for adding two types of access tokens to requests.
 
 - Bearer http://tools.ietf.org/html/rfc6750
 - MAC http://tools.ietf.org/html/draft-ietf-oauth-v2-http-mac-01
-
 """
 from __future__ import absolute_import, unicode_literals
 
@@ -20,9 +19,55 @@ except ImportError:
 
 from oauthlib.common import add_params_to_uri, add_params_to_qs, unicode_type
 from oauthlib import common
+
 from . import utils
 
 
+class OAuth2Token(dict):
+
+    def __init__(self, params, old_scope=None):
+        super(OAuth2Token, self).__init__(params)
+        self._new_scope = None
+        if 'scope' in params and params['scope']:
+            self._new_scope = set(utils.scope_to_list(params['scope']))
+        if old_scope is not None:
+            self._old_scope = set(utils.scope_to_list(old_scope))
+            if self._new_scope is None:
+                # the rfc says that if the scope hasn't changed, it's optional
+                # in params so set the new scope to the old scope
+                self._new_scope = self._old_scope
+        else:
+            self._old_scope = self._new_scope
+
+    @property
+    def scope_changed(self):
+        return self._new_scope != self._old_scope
+
+    @property
+    def old_scope(self):
+        return utils.list_to_scope(self._old_scope)
+
+    @property
+    def old_scopes(self):
+        return list(self._old_scope)
+
+    @property
+    def scope(self):
+        return utils.list_to_scope(self._new_scope)
+
+    @property
+    def scopes(self):
+        return list(self._new_scope)
+
+    @property
+    def missing_scopes(self):
+        return list(self._old_scope - self._new_scope)
+
+    @property
+    def additional_scopes(self):
+        return list(self._new_scope - self._old_scope)
+
+
 def prepare_mac_header(token, uri, key, http_method,
                        nonce=None,
                        headers=None,
@@ -187,6 +232,10 @@ class TokenBase(object):
 
 
 class BearerToken(TokenBase):
+    __slots__ = (
+        'request_validator', 'token_generator',
+        'refresh_token_generator', 'expires_in'
+    )
 
     def __init__(self, request_validator=None, token_generator=None,
                  expires_in=None, refresh_token_generator=None):
@@ -197,7 +246,7 @@ class BearerToken(TokenBase):
         )
         self.expires_in = expires_in or 3600
 
-    def create_token(self, request, refresh_token=False):
+    def create_token(self, request, refresh_token=False, save_token=True):
         """Create a BearerToken, by default without refresh token."""
 
         if callable(self.expires_in):
@@ -213,6 +262,9 @@ class BearerToken(TokenBase):
             'token_type': 'Bearer',
         }
 
+        # If provided, include - this is optional in some cases https://tools.ietf.org/html/rfc6749#section-3.3 but
+        # there is currently no mechanism to coordinate issuing a token for only a subset of the requested scopes so
+        # all tokens issued are for the entire set of requested scopes.
         if request.scopes is not None:
             token['scope'] = ' '.join(request.scopes)
 
@@ -227,8 +279,9 @@ class BearerToken(TokenBase):
                 token['refresh_token'] = self.refresh_token_generator(request)
 
         token.update(request.extra_credentials or {})
-
-        self.request_validator.save_bearer_token(token, request)
+        token = OAuth2Token(token)
+        if save_token:
+            self.request_validator.save_bearer_token(token, request)
         return token
 
     def validate_request(self, request):
